import type { Options, SendCommandFn } from './types';
import type { RateLimiter, RateLimiterIncrementResult, RateLimiterOptions } from './RateLimiter';
/**
 * A `Store` for the `express-rate-limit` package that stores hit counts in
 * Redis.
 */
declare class RedisStore implements RateLimiter {
    /**
     * The function used to send raw commands to Redis.
     */
    sendCommand: SendCommandFn;
    /**
     * The text to prepend to the key in Redis.
     */
    prefix: string;
    /**
     * Whether to reset the expiry for a particular key whenever its hit count
     * changes.
     */
    resetExpiryOnChange: boolean;
    /**
     * Stores the loaded SHA1 of the LUA script for executing the increment operations.
     */
    loadedScriptSha1: Promise<string>;
    /**
     * The number of milliseconds to remember that user's requests.
     */
    windowMs: number;
    /**
     * @constructor for `RedisStore`.
     *
     * @param options {Options} - The configuration options for the store.
     */
    constructor(options: Options);
    /**
     * Setup the RedisRateLimitStore.
     * Must be called before first use.
     */
    setup(): Promise<void>;
    loadScript(): Promise<string>;
    /**
     * Method to prefix the keys with the given text.
     *
     * @param key {string} - The key.
     *
     * @returns {string} - The text + the key.
     */
    prefixKey(key: string): string;
    /**
     * Method that actually initializes the store.
     *
     * @param options {RateLimitConfiguration} - The options used to setup the middleware.
     */
    init(options: RateLimiterOptions): void;
    /**
     * Method to increment a client's hit counter.
     *
     * @param key {string} - The identifier for a client
     */
    increment(key: string, amount?: number): Promise<RateLimiterIncrementResult>;
    /**
     * Method to decrement a client's hit counter.
     *
     * @param key {string} - The identifier for a client
     */
    decrement(key: string, amount?: number): Promise<void>;
    /**
     * Method to reset a client's hit counter.
     *
     * @param key {string} - The identifier for a client
     */
    resetKey(key: string): Promise<void>;
    private _runScript;
}
export default RedisStore;
//# sourceMappingURL=lib.d.ts.map