var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * A `Store` for the `express-rate-limit` package that stores hit counts in
 * Redis.
 */
class RedisStore {
    /**
     * @constructor for `RedisStore`.
     *
     * @param options {Options} - The configuration options for the store.
     */
    constructor(options) {
        var _a, _b;
        this.sendCommand = options.sendCommand;
        this.prefix = (_a = options.prefix) !== null && _a !== void 0 ? _a : 'rl:';
        this.resetExpiryOnChange = (_b = options.resetExpiryOnChange) !== null && _b !== void 0 ? _b : false;
    }
    /**
     * Setup the RedisRateLimitStore.
     * Must be called before first use.
     */
    setup() {
        return __awaiter(this, void 0, void 0, function* () {
            // So that the script loading can occur non-blocking, this will send
            // the script to be loaded, and will capture the value within the
            // promise return. This way, if increments start being called before
            // the script has finished loading, it will wait until it is loaded
            // before it continues.
            this.loadedScriptSha1 = this.loadScript();
        });
    }
    loadScript() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.sendCommand('SCRIPT', 'LOAD', `
        local count = tonumber(ARGV[3])
        local totalHits = redis.call("INCRBY", KEYS[1], count)
        local timeToExpire = redis.call("PTTL", KEYS[1])
        if timeToExpire <= 0 or ARGV[1] == "1"
        then
            redis.call("PEXPIRE", KEYS[1], tonumber(ARGV[2]))
            timeToExpire = tonumber(ARGV[2])
        end

        return { totalHits, timeToExpire }
    `
                // Ensure that code changes that affect whitespace do not affect
                // the script contents.
                .replace(/^\s+/gm, '')
                .trim());
            if (typeof result !== 'string') {
                throw new TypeError('unexpected reply from redis client');
            }
            return result;
        });
    }
    /**
     * Method to prefix the keys with the given text.
     *
     * @param key {string} - The key.
     *
     * @returns {string} - The text + the key.
     */
    prefixKey(key) {
        return `${this.prefix}${key}`;
    }
    /**
     * Method that actually initializes the store.
     *
     * @param options {RateLimitConfiguration} - The options used to setup the middleware.
     */
    init(options) {
        this.windowMs = options.windowMs;
    }
    /**
     * Method to increment a client's hit counter.
     *
     * @param key {string} - The identifier for a client
     */
    increment(key, amount = 1) {
        return __awaiter(this, void 0, void 0, function* () {
            const results = yield this._runScript(this.prefixKey(key), amount);
            if (!Array.isArray(results)) {
                throw new TypeError('Expected result to be array of values');
            }
            if (results.length !== 2) {
                throw new Error(`Expected 2 replies, got ${results.length}`);
            }
            const totalHits = results[0];
            if (typeof totalHits !== 'number') {
                throw new TypeError('Expected value to be a number');
            }
            const timeToExpire = results[1];
            if (typeof timeToExpire !== 'number') {
                throw new TypeError('Expected value to be a number');
            }
            const resetTimeMs = Date.now() + timeToExpire;
            return {
                totalHits,
                resetTimeMs,
            };
        });
    }
    /**
     * Method to decrement a client's hit counter.
     *
     * @param key {string} - The identifier for a client
     */
    decrement(key, amount = 1) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.sendCommand('DECRBY', this.prefixKey(key), amount.toString());
        });
    }
    /**
     * Method to reset a client's hit counter.
     *
     * @param key {string} - The identifier for a client
     */
    resetKey(key) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.sendCommand('DEL', this.prefixKey(key));
        });
    }
    _runScript(key, amount) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.sendCommand('EVALSHA', yield this.loadedScriptSha1, '1', key, this.resetExpiryOnChange ? '1' : '0', this.windowMs.toString(), amount.toString());
            }
            catch (e) {
                const errString = e.toString();
                if (errString.includes('NOSCRIPT')) {
                    this.loadedScriptSha1 = this.loadScript();
                    return yield this._runScript(key, amount);
                }
                throw e;
            }
        });
    }
}
export default RedisStore;
//# sourceMappingURL=lib.js.map